@@ -39,6 +39,7 @@
 #include "simfs.h"
 #include "simutil.h"
 #include "storage.h"
+#include "missing.h"
 
 #define SIM_CACHE_MODE 0600
 #define SIM_CACHE_BASEPATH STORAGEDIR "/%s-%i"
@@ -274,7 +275,7 @@ static void sim_fs_end_current(struct sim_fs *fs)
 		__ofono_sim_remove_session_watch(fs->session, fs->watch_id);
 
 	if (fs->fd != -1) {
-		TFR(close(fs->fd));
+		L_TFR(close(fs->fd));
 		fs->fd = -1;
 	}
 
@@ -320,7 +321,7 @@ static gboolean cache_block(struct sim_fs *fs, int block, int block_len,
 				SIM_CACHE_HEADER_SIZE, SEEK_SET) == (off_t) -1)
 		return FALSE;
 
-	r = TFR(write(fs->fd, data, num_bytes));
+	r = L_TFR(write(fs->fd, data, num_bytes));
 
 	if (r != num_bytes)
 		return FALSE;
@@ -335,7 +336,7 @@ static gboolean cache_block(struct sim_fs *fs, int block, int block_len,
 	b = fs->bitmap[offset];
 	b |= 1 << bit;
 
-	r = TFR(write(fs->fd, &b, sizeof(b)));
+	r = L_TFR(write(fs->fd, &b, sizeof(b)));
 
 	if (r != sizeof(b))
 		return FALSE;
@@ -477,7 +478,7 @@ static gboolean sim_fs_op_read_block(gpointer user_data)
 		if (lseek(fs->fd, seekoff, SEEK_SET) == (off_t) -1)
 			break;
 
-		if (TFR(read(fs->fd, op->buffer + bufoff, toread)) != toread)
+		if (L_TFR(read(fs->fd, op->buffer + bufoff, toread)) != toread)
 			break;
 
 		op->current += 1;
@@ -569,7 +570,7 @@ static gboolean sim_fs_op_read_record(gpointer user)
 				SIM_CACHE_HEADER_SIZE, SEEK_SET) == (off_t) -1)
 			break;
 
-		if (TFR(read(fs->fd, buf, op->record_length)) !=
+		if (L_TFR(read(fs->fd, buf, op->record_length)) !=
 				op->record_length)
 			break;
 
@@ -660,17 +661,17 @@ static void sim_fs_op_cache_fileinfo(struct sim_fs *fs,
 	fileinfo[6] = file_status;
 
 	path = g_strdup_printf(SIM_CACHE_PATH, imsi, phase, op->id);
-	fs->fd = TFR(open(path, O_WRONLY | O_CREAT | O_TRUNC, SIM_CACHE_MODE));
+	fs->fd = L_TFR(open(path, O_WRONLY | O_CREAT | O_TRUNC, SIM_CACHE_MODE));
 	g_free(path);
 
 	if (fs->fd == -1)
 		return;
 
-	if (TFR(write(fs->fd, fileinfo, SIM_CACHE_HEADER_SIZE)) ==
+	if (L_TFR(write(fs->fd, fileinfo, SIM_CACHE_HEADER_SIZE)) ==
 			SIM_CACHE_HEADER_SIZE)
 		return;
 
-	TFR(close(fs->fd));
+	L_TFR(close(fs->fd));
 	fs->fd = -1;
 }
 
@@ -761,7 +762,7 @@ static gboolean sim_fs_op_check_cached(struct sim_fs *fs)
 	if (path == NULL)
 		return FALSE;
 
-	fd = TFR(open(path, O_RDWR));
+	fd = L_TFR(open(path, O_RDWR));
 	g_free(path);
 
 	if (fd == -1) {
@@ -773,7 +774,7 @@ static gboolean sim_fs_op_check_cached(struct sim_fs *fs)
 		return FALSE;
 	}
 
-	len = TFR(read(fd, fileinfo, SIM_CACHE_HEADER_SIZE));
+	len = L_TFR(read(fd, fileinfo, SIM_CACHE_HEADER_SIZE));
 
 	if (len != SIM_CACHE_HEADER_SIZE)
 		goto error;
@@ -827,7 +828,7 @@ static gboolean sim_fs_op_check_cached(struct sim_fs *fs)
 	return TRUE;
 
 error:
-	TFR(close(fd));
+	L_TFR(close(fd));
 	return FALSE;
 }
 
@@ -1128,7 +1129,7 @@ int sim_fs_write(struct ofono_sim_context *context, int id,
 	op->cb = cb;
 	op->userdata = userdata;
 	op->is_read = FALSE;
-	op->buffer = g_memdup(data, length);
+	op->buffer = g_memdup2(data, length);
 	op->structure = structure;
 	op->length = length;
 	op->current = record;
@@ -1187,8 +1188,8 @@ char *sim_fs_get_cached_image(struct sim_fs *fs, int id)
 
 	path = g_strdup_printf(SIM_IMAGE_CACHE_PATH, imsi, phase, id);
 
-	TFR(stat(path, &st_buf));
-	fd = TFR(open(path, O_RDONLY));
+	L_TFR(stat(path, &st_buf));
+	fd = L_TFR(open(path, O_RDONLY));
 	g_free(path);
 
 	if (fd < 0)
@@ -1198,12 +1199,12 @@ char *sim_fs_get_cached_image(struct sim_fs *fs, int id)
 	buffer = g_try_new0(char, image_length + 1);
 
 	if (buffer == NULL) {
-		TFR(close(fd));
+		L_TFR(close(fd));
 		return NULL;
 	}
 
-	len = TFR(read(fd, buffer, image_length));
-	TFR(close(fd));
+	len = L_TFR(read(fd, buffer, image_length));
+	L_TFR(close(fd));
 
 	if (len != image_length) {
 		g_free(buffer);
