@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <assert.h>
 #include <glib.h>
+#include <ell/ell.h>
 
 #include <ofono/types.h>
 
@@ -85,6 +86,27 @@ static void test_ber_tlv_iter(void)
 	test_buffer(valid_mms_params, sizeof(valid_mms_params));
 }
 
+static void test_validate_tlv(void)
+{
+	unsigned char impi_none[] = { 0x80, 0x0 };
+	unsigned char impi_empty[] = { 0x80, 0x1, '\0' };
+	unsigned char impi_term1[] = { 0x80, 0x3, 'F', 'O', 'O' };
+	unsigned char impi_term2[] = { 0x80, 0x4, 'F', 'O', 'O', '\0' };
+	unsigned char impi_term3[] = { 0x80, 0x3, 'F', 'O', 'O', 0xff, 0xff };
+	unsigned char impi_term4[] = { 0x80, 0x4, 'F', 'O', 'O', '\0', 0xff };
+	unsigned char impi_invalid1[] = { 0x80, 0x4, 'F', '\0', 'O', '\0' };
+	unsigned char impi_invalid2[] = { 0x80, 0x4, 0xff, 0xff, 0xff, 0xff };
+
+	g_assert(validate_utf8_tlv(impi_none) == FALSE);
+	g_assert(validate_utf8_tlv(impi_empty) == TRUE);
+	g_assert(validate_utf8_tlv(impi_term1) == TRUE);
+	g_assert(validate_utf8_tlv(impi_term2) == TRUE);
+	g_assert(validate_utf8_tlv(impi_term3) == TRUE);
+	g_assert(validate_utf8_tlv(impi_term4) == TRUE);
+	g_assert(validate_utf8_tlv(impi_invalid1) == FALSE);
+	g_assert(validate_utf8_tlv(impi_invalid2) == FALSE);
+}
+
 static void test_ber_tlv_builder_mms(void)
 {
 	struct ber_tlv_iter top_iter, nested_iter;
@@ -407,12 +429,12 @@ static const char *record_ef = "62198205422100200483026F408A01058B036F0607"
 static void test_3g_status_data(void)
 {
 	unsigned char *response;
-	long len;
+	size_t len;
 	int flen, rlen, str;
 	unsigned char access[3];
 	unsigned short efid;
 
-	response = decode_hex(binary_ef, -1, &len, 0);
+	response = l_util_from_hexstring(binary_ef, &len);
 
 	sim_parse_3g_get_response(response, len, &flen, &rlen, &str,
 					access, &efid);
@@ -427,7 +449,7 @@ static void test_3g_status_data(void)
 
 	g_free(response);
 
-	response = decode_hex(record_ef, -1, &len, 0);
+	response = l_util_from_hexstring(record_ef, &len);
 
 	sim_parse_3g_get_response(response, len, &flen, &rlen, &str,
 					access, &efid);
@@ -450,11 +472,11 @@ static char *at_cuad_response = "611B4F10A0000000871002FFFFFFFF8905080000"
 static void test_application_entry_decode(void)
 {
 	unsigned char *ef_dir;
-	long len;
+	size_t len;
 	GSList *entries;
 	struct sim_app_record *app[2];
 
-	ef_dir = decode_hex(at_cuad_response, -1, &len, 0);
+	ef_dir = l_util_from_hexstring(at_cuad_response, &len);
 	entries = sim_parse_app_template_entries(ef_dir, len);
 
 	g_assert(g_slist_length(entries) == 2);
@@ -471,6 +493,7 @@ static void test_application_entry_decode(void)
 	g_assert(app[1]->label != NULL);
 	g_assert(!strcmp(app[1]->label, "MIDPfiles"));
 
+	g_slist_free_full(entries, (GDestroyNotify) sim_app_record_free);
 	g_free(ef_dir);
 }
 
@@ -608,6 +631,7 @@ int main(int argc, char **argv)
 	g_test_init(&argc, &argv, NULL);
 
 	g_test_add_func("/testsimutil/ber tlv iter", test_ber_tlv_iter);
+	g_test_add_func("/testsimutil/ber tlv validate utf8", test_validate_tlv);
 	g_test_add_func("/testsimutil/ber tlv encode MMS",
 			test_ber_tlv_builder_mms);
 	g_test_add_func("/testsimutil/ber tlv encode EFpnn",
